% function to produce an image of the fractal developed when using newton's
% method to find roots given starting points. takes a polynomial, performs
% N newton iterations over a grid in the complex plane and produces a
% colormap based on the results of the iterations at that point

% p - a row vector of the input polynomial's coefficients ie [1 0 0 -1] =
% z^3 -1
% xmin, xmax, ymin, ymax - screen bounds
% tol1 - float, typically of the format 1ex where x is an integer.
% digits of accuracy in the rounding. for example, an input of 1e-3
% rounds the result of the newton iterations to 3 decimal places and rounds
% the results of the call to root to 3 decimal places in order to compare
% if a function converged to a root
% tol2 - float, typically of the format 1ex where x is an integer.
%stops performing calculations a number when the different between
% iterations is under this tolerance.
% step - how fine the grid is. "resolution"
% N - integer, number of newton iterations to perform
% m - coefficient to multiplty by for the relaxed newton's method. keep
% this at 1 for the regular newton's method
% mr - boolean, perform calculations using newton's method for multiple roots instead
% centerize - boolean, centerize the data before plotting the image.
% usually the matrix will be a bunch of integers. this centerizes the data
% about the mean.
% iterationscale - boolean, display graphic color is based on root converged to AND
% number of iterations until convergence
% onlyiteration - boolean, display graphic is colored based only on number of
% iterations until convergence
% binaryimg - boolean, if true output will be a matrix of all ones and zeros.
% use to display the fractal in isolation.
% critical - integer. argument that's passed to the newtp function that
% determines at which iteration it should switch over to only doing newton iterations on
% converged value 
% dotime - boolean, display the time in each iteration of the newton
% iteration loop
% scale - float, scales the axes by this number
% random - integer. if nonzero uses a random polynomial of degree given by random in order to generate the fractal 
function r = fractalp(varargin)

    % this code was taken from RazerM of stack exchange. thanks RazerM
    %# define defaults at the beginning of the code so that you do not need to
    %# scroll way down in case you want to change something or if the help is
    %# incomplete
    options = struct('p', [1 0 0 -1],'xmin',-2,'xmax',2,'ymin',-2,'ymax',2,'step',.01,'n',100,'tol1',1e-3,'tol2',1e-4,'m',1,'mr',false,'old',false,'cent',true,'iterationscale',false,'onlyiteration',false,'binaryimg',false,'critical',10,'dotime',false,'scale',1,'random',0);

    %# read the acceptable names
    optionNames = fieldnames(options);

    %# count arguments
    nArgs = length(varargin);
    if round(nArgs/2)~=nArgs/2
        error('EXAMPLE needs propertyName/propertyValue pairs')
    end

    for pair = reshape(varargin,2,[]) %# pair is {propName;propValue}
        inpName = lower(pair{1}); %# make case insensitive

        if any(strmatch(inpName,optionNames))
            %# overwrite options. If you want you can test for the right class here
            %# Also, if you find out that there is an option you keep getting wrong,
            %# you can use "if strcmp(inpName,'problemOption'),testMore,end"-statements
            options.(inpName) = pair{2};
        else
            error('%s is not a recognized parameter name',inpName)
        end
    end

    % set up the grid:
    scale = options.('scale');
    xmin = options.('xmin')*scale;
    xmax = options.('xmax')*scale;
    ymin = options.('ymin')*scale;
    ymax = options.('ymax')*scale;
    x = xmin:options.('step'):xmax;
    y = ymin:options.('step'):ymax;
    [X,Y] = meshgrid(x, y);
    
    % if random was enabled create the random polynomial
    if options.('random') > 0
        options.('p') = 5*randn(1,options.('random')+1) + 5*i*randn(1,options.('random')+1);
    end
    
    % find out which roots each complex number converges to
    if options.('old') % constant time per iteration
        Z = oldnewtp(options.('p'), X + Y*i, options.('n'), options.('m'),options.('dotime'));
    else % quicker time per iteration
        [Z,S] = newtp(options.('p'), X + Y*i, options.('n'), options.('m'), options.('tol2'), options.('mr'),options.('critical'),options.('dotime'));
    end
    % round each numbers
    Z = round(Z * 1/options.('tol1'))/options.('tol1');
    
    % determine all roots of the function and store them in an array
    r = roots(options.('p'));
    n = length(r);
    
    % round each root to 8 decimal places
    r = round(r * 1/options.('tol1'))/1/options.('tol1');
    
    % add a zero in the end
    if ~ismember(0, r)
        r = [r.' 0].';
    end
    
    % create a matrix full of the indices to which the number converged
    [~, loc] = ismember(Z,r);
    
    if options.('cent') loc = (loc - n/2)/(n/2); end
    
   
    
    if options.('binaryimg')
        r = tobinary(loc);
        
        imagesc(x,y,r);
        colormap gray;
    elseif options.('onlyiteration')
        imagesc(x,y,S);
    else
        r = loc;
        if options.('iterationscale')
            S = S/max(S(:));
            loc = loc.*S;
        end
        imagesc(x,y,loc);
    end
    xlabel('Re(z)')
    ylabel('Im(z)')
    axis square
    options
end